-- Type: tprintrcv

DROP TYPE tprintrcv CASCADE;

CREATE TYPE tprintrcv AS
   (saledate date,
    discamt numeric(19,4),
    discexp character varying,
    taxamt numeric(19,4),
    crtby character varying,
    cmpname character varying,
    cmpaddr character varying,
    cmpphone character varying,
    cmpcity character varying,
    cmpfax character varying,
    totspaid numeric(19,4),
    totrcvamt numeric(19,4),
    totchgamt numeric(19,4),
    overrcv numeric(19,4),
    surcamt numeric(19,4),
    surcnote character varying,
    leavetime time without time zone,
    subtotal numeric(19,4),
    shipto character varying,
    hostdesc character varying,
    custname character varying,
    billno character varying,
    bpphone character varying,
    crtdate timestamp without time zone,
    billnote character varying,
    posdesc character varying,
    servicecharge numeric(19,4),
    totalamt numeric(19,4),
    dpamt numeric(19,4),
    dueamt numeric(19,4),
    cash numeric(19,4),
    credit numeric(19,4),
    debit numeric(19,4),
    voucher numeric(19,4),
    change numeric(19,4),
    nontunai numeric(19,4),
    nontunaivouc numeric(19,4));
ALTER TYPE tprintrcv
  OWNER TO bits;

-- Function: fprintrcv(character varying, character varying)

-- DROP FUNCTION fprintrcv(character varying, character varying);

CREATE OR REPLACE FUNCTION fprintrcv(_saleno character varying, _rcvno character varying)
  RETURNS SETOF tprintrcv AS
$BODY$
DECLARE
	_retval 		tPrintRcv%ROWTYPE;
	_surcamt_total 	NUMERIC(19,4);
	_surcnote_total VARCHAR;
	_surcamt 		NUMERIC(19,4);
	_surcnote 		VARCHAR;
	_rcvdamt 		rcvd.rcvdamt%TYPE;
	_rcvdmtd		rcvd.rcvdmtd%TYPE;
	_cash			NUMERIC(19,4);
	_credit			NUMERIC(19,4);
	_debit			NUMERIC(19,4);
	_vouc			NUMERIC(19,4);
	_change			NUMERIC(19,4);
	cData			REFCURSOR;
BEGIN
	SELECT INTO 
		_retval.saledate, _retval.crtby, _retval.discamt, _retval.taxamt, _retval.discexp, _retval.subtotal, _retval.custname, _retval.posdesc, _retval.servicecharge
		saledate, crtby, discamt, taxamt, discexp, subtotal, b.bpname, p.posdesc, s.scamt
	FROM sale s
	JOIN bp b ON b.bpid=s.custid
	JOIN pos p ON p.posid=s.posid
	WHERE saleno=_SaleNo;

	SELECT INTO 
		_retval.cmpname, _retval.cmpaddr, _retval.cmpphone, _retval.cmpcity, _retval.cmpfax
		cmpname, cmpaddr, phone, cityname, fax 
	FROM cmp 
	LEFT JOIN city ON city.cityid=cmp.cityid;
	
	SELECT INTO 
		_retval.overrcv 
		abs(rcvdamt)
	FROM rcvd 
	WHERE rcvdamt<0 AND rcvno=_RcvNo ORDER BY rcvdno DESC LIMIT 1;

	IF NOT FOUND THEN
		_retval.overrcv=0;
	END IF;

	SELECT INTO 
		_retval.totspaid, _retval.totrcvamt, _retval.leavetime, _retval.crtdate
		totspaid, totrcvamt, crtdate::timestamp::time, crtdate::timestamp
	FROM rcv 
	WHERE rcvno=_RcvNo;

	SELECT INTO _retval.totrcvamt SUM(ABS(rcvdamt*excrate)) 
	FROM rcvd WHERE rcvno=_RcvNo AND rcvdamt>0;

	SELECT INTO _retval.totchgamt SUM(ABS(rcvdamt*excrate)) 
	FROM rcvd WHERE rcvno=_RcvNo AND rcvdamt<0;
	
	_surcamt_total := 0;
	_surcnote_total := '';
	
	OPEN cData FOR
		SELECT spaidamt, spaidnote
		FROM spaid WHERE rcvno=_RcvNo AND spaidno>1 AND spaidamt<0 AND issurcharge IS TRUE;
	LOOP
		FETCH cData INTO _surcamt, _surcnote;
		EXIT WHEN NOT FOUND;
		
		_surcamt_total := _surcamt_total+_surcamt;
		
		IF _surcnote_total='' THEN
			_surcnote_total := _surcnote;
		ELSE
			_surcnote_total := _surcnote_total || ' | ' || _surcnote;
		END IF;
		
	END LOOP;
	CLOSE cData;
	
	_surcamt_total := _surcamt_total*-1;
	
	_retval.surcamt := _surcamt_total;
	_retval.surcnote := _surcnote_total;
	_retval.totspaid := _retval.totspaid+_surcamt_total;
	_retval.totrcvamt := _retval.totrcvamt+_surcamt_total;

	SELECT INTO _retval.hostdesc, _retval.billno, _retval.billnote, _retval.shipto, _retval.bpphone
		hostdesc, bill.billno, billnote, bill.shipto, bpphone
	FROM bill JOIN billsale ON bill.billno=billsale.billno 
	LEFT JOIN host ON bill.hostid=host.hostid
	WHERE saleno=_SaleNo;
	
	SELECT INTO _retval.totalamt spaidamt
	FROM spaid
	WHERE refno=_SaleNo AND reftype='SALE';
	
	SELECT INTO _retval.dpamt COALESCE(SUM(dpamt),0)
	FROM dp
	WHERE refno=_retval.billno AND reftype='BILL';
	
	_retval.totalamt := _retval.totalamt+_surcamt_total;
	_retval.dueamt := _retval.totalamt-_retval.dpamt;

	OPEN cData FOR
		SELECT rcvdmtd, SUM(rcvdamt) FROM rcvd WHERE rcvno = _rcvno AND rcvdamt > 0
		GROUP BY rcvdmtd;
	LOOP
		FETCH cData INTO _rcvdmtd, _rcvdamt;
		EXIT WHEN NOT FOUND;
		
		IF _rcvdmtd = 'CASH' THEN
			_cash := _rcvdamt;
		ELSEIF _rcvdmtd = 'CRC' THEN
			_credit := _rcvdamt;
		ELSEIF _rcvdmtd = 'DBC' THEN
			_debit := _rcvdamt;
		ELSEIF _rcvdmtd = 'VOUC' THEN
			_vouc := _rcvdamt;
		END IF;
		
	END LOOP;
	CLOSE cData;

	_retval.cash := COALESCE(_cash, 0);
	_retval.credit := COALESCE(_credit, 0);
	_retval.debit := COALESCE(_debit, 0);
	_retval.voucher := COALESCE(_vouc, 0);
	_retval.nontunai := COALESCE(_credit, 0) + COALESCE(_debit, 0);
	_retval.nontunaivouc := COALESCE(_credit, 0) + COALESCE(_debit, 0) + COALESCE(_vouc, 0);
	

	_change := 0;
	OPEN cData FOR
		SELECT rcvdamt FROM rcvd WHERE rcvno = _rcvno;
	LOOP
		FETCH cData INTO _rcvdamt;
		EXIT WHEN NOT FOUND;
		
		IF _rcvdamt < 0 THEN
			_change := _change + ABS(_rcvdamt);
		END IF;
		
	END LOOP;
	CLOSE cData;

	_retval.change := COALESCE(_change, 0);

	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION fprintrcv(character varying, character varying)
  OWNER TO bits;

